﻿cDisplay = 
{  
    ---
    -- A shortcut to add a message to the default chat window
    printStyle = {
        white = { r=1.0, g=1.0, b=1.0, group=54, addToStart=false },
        gray = { r=0.5, g=0.5, b=0.5, group=53, addToStart=false }
    },
    Print = function(self, message, style)
        if style == nil then
            style = self.printStyle.white
        end
        DEFAULT_CHAT_FRAME:AddMessage(message, style.r, style.g, style.b, style.group, style.addToStart);
    end,

    ---
    -- Average Window controls
    ---
    Average = 
    {
        window = nil,
        lines = {},
        backdrop = nil,
    
        -- Window Style
        visible = true, -- Note that this should be treated as a private member. Use the SetVisibility function.
        showBackground = true,
        textMargin = 5,
        lineSpacing = 2,
        
        ---
        -- Creates the window
        Initialize = function(self)
            if self.window ~= nil then
                return
            end
        
            self.window = XToLevelDisplay
            self.backdrop = self.window:GetBackdrop();
            
            -- Create the lines
            self.lines.header = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_h1')
            self.lines.header:SetText('XToLevel')
            self.lines.header.group = "header"
            self.lines.header.tabIndex = 1
            
            self.lines.playerKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerKills:SetText(L["Kills"] .. ': ...')
            self.lines.playerKills.group = "player"
            self.lines.playerKills.tabIndex = 2
            
            self.lines.playerQuests = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerQuests:SetText(L["Quests"] .. ': ...')
            self.lines.playerQuests.group = "player"
            self.lines.playerQuests.tabIndex = 3
			
			self.lines.petKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.petKills:SetText(L["Pet"] .. ': ...')
            self.lines.petKills.group = "pet"
            self.lines.petKills.tabIndex = 4
            
            self:Update()
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the window
        Update = function(self)
            if self.window == nil then
                return
            end
            
            if not sConfig.averageDisplay.visible then
                self.window:Hide()
                return
            elseif not cPlayer.isActive and not cPet.isActive then
				self.window:Hide()
                return
			else
				self.window:Show()
            end
			
			if sConfig.averageDisplay.header then
				self.lines.header:Show()
			end
			
			if not cPlayer.isActive then
				self.lines.playerKills:Hide()
				self.lines.playerQuests:Hide()
			else
				self.lines.playerKills:Show()
				self.lines.playerQuests:Show()
			end
			if not cPet.isActive then
				self.lines.petKills:Hide()
			else
				self.lines.petKills:Show()
			end
			
            -- Show / Hide the lines and update the anchors
            -- The elements are re-arraged into a temp array according to their tabIndex
            local iLines = {}
            for eName, eElem in pairs(self.lines) do
                iLines[eElem.tabIndex] = { name=eName, elem=eElem }
            end
            
            local nextAnchor = self.window
            local nextMarginV = self.textMargin
            local nextMarginH = self.textMargin
            local nextPoint = "TOPLEFT"
            local currentIndent =  -nextMarginH
            for index, value in ipairs(iLines) do
                local name = value.name
                local elem = value.elem
                
                elem:ClearAllPoints()
                if sConfig.averageDisplay[name] and elem:IsShown() then
                    -- Clear text indent if the previous element was in the same group
                    if nextAnchor.group == elem.group then
                        nextMarginH = 0
                    else
                        currentIndent = currentIndent + nextMarginH
                    end
                    elem:SetPoint("TOPLEFT", nextAnchor, nextPoint, nextMarginH, -nextMarginV)
                    elem.lineIndent = currentIndent
                    
                    nextAnchor = elem
                    nextMarginV = self.lineSpacing
                    nextPoint = "BOTTOMLEFT"
                else
                    elem:Hide()
                end
            end
            
            -- Update messages for the visible elements
            if self.lines.header:IsShown() then
                self.lines.header:SetText('XToLevel')
            end
			
			if cPlayer.isActive then
				if self.lines.playerKills:IsShown() then
					self.lines.playerKills:SetText(L["Kills"] .. ': ' .. cPlayer:GetAverageKillsRemaining())
				end
				if self.lines.playerQuests:IsShown() then
					self.lines.playerQuests:SetText(L["Quests"] .. ': ' .. cPlayer:GetAverageQuestsRemaining())
				end
			end
			
			if cPet.isActive then
				if self.lines.petKills:IsShown() then
					self.lines.petKills:SetText(L["Pet"] .. ': ' .. cPet:GetAverageKillsRemaining())
				end
			end
            
            -- Calculate the height and width of the lines
            local maxWidth = 0
            local totalHeight = self.textMargin * 2
            for name, value in pairs(self.lines) do
                if value:IsVisible() then
                    local currentWidth = value:GetWidth() + (value.lineIndent or 0)
                    if currentWidth  > maxWidth then
                        maxWidth = currentWidth
                    end
                    totalHeight = totalHeight + value:GetHeight() + self.lineSpacing
                end
            end
            local totalWidth = maxWidth + (self.textMargin * 2)
            
            -- Set the display box size
            self.window:SetWidth(totalWidth)
            self.window:SetHeight(totalHeight)
            
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the backdrop
        updateBackdrop = function(self)
            if self.window == nil then
                return
            end
            
            if sConfig.averageDisplay.backdrop then
                self.window:SetBackdrop(self.backdrop);
                self.window:SetBackdropColor("0.0", "0.75", "0.5", "0.75");
                self.window:SetBackdropBorderColor("0.0", "0.0", "0.0", "1.0");
            else
                self.window:SetBackdrop(nil);
            end
        end,
        
        -- Show / Hide the window
        ToggleVisibility = function(self, newValue)
            if self.window == nil then
                return
            end
        
            if(newValue) then
                self.visible = newValue
            else
                self.visible = not self.visible
            end
            
            if self.visible then
                self.window:Show()
            else
                self.window:Hide()
            end
        end
    },
    
    ---
    -- Controls for the floating display
    -- The settings check is added here to simplify the calling code.
    ---
    Floating = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5 },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, fade=6 },
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerFloating then
                local message = mobsRequired .." ".. mobName .. L["Kills Needed"]
                self:Print(message, self.killStyle)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petFloating then -- sConfig.messages.petFloating
                local message = mobsRequired .. " " .. mobName .. L["Pet Kills Needed"] .. petName
                self:Print(message, self.petStyle)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerFloating then
                local message = questsRequired .. L["Quests Needed"]
                self:Print(message, self.questStyle)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerFloating then
                local message = L["Level Reached"]
                self:Print(message, self.levelStyle)
            end
        end,
        
        Print = function(self, text, style)
            UIErrorsFrame:AddMessage(text, style.r, style.g, style.b, style.group, style.fade);
        end
    },
    
    ---
    -- Controls for the chat display
    ---
    Chat = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5, addToStart=false },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, addToStart=false},
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerChat then
                local message = mobsRequired .." ".. mobName .. L["Kills Needed"]
                cDisplay:Print(message, self.killStyle)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petChat then -- sConfig.messages.petFloating
                local message = mobsRequired .. " " .. mobName .. L["Pet Kills Needed"] .. petName
                cDisplay:Print(message, self.petStyle)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerChat then
                local message = questsRequired .. L["Quests Needed"]
                cDisplay:Print(message, self.questStyle)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerChat then
                local message = L["Level Reached"]
                cDisplay:Print(message, self.levelStyle)
            end
        end
    },
    
    ---
    -- Controls for the configuration window
    ---
    Config =
    {
        waterfall = nil,
		
		Initialize = function(self)
			self.waterfall = AceLibrary("Waterfall-1.0")
            self.waterfall:Register('ConfigWindow',
				'title',L["Config Header"],
 				'aceOptions', self:getOptions(),
				'treeLevels', 3,
				'colorR', 0.0,
                'colorG', 0.5,
                'colroB', 0.25
			);
		end,
		
		Open = function(self, page)
			if page == nil then page = "general" end
			self.waterfall:Open('ConfigWindow', page)
		end,
		
		getOptions = function(self)
			return {
				type = 'group',
				name = L["XToLevel"],
				desc = L["Config Header"],
				order = 1,
				args = {
					general = {
						type = 'group',
						name = L["General Name"],
						order = 100,
						args = {
							localeHeader = {
								type = 'header',
								name = L["Locale Header"],
								order = 101,
							},
							localeDropdown = {
								type = 'text',
								name = L["Display Locale"],
								get = function() return sConfig.general.displayLocale or GetLocale(); end,
								set = function(value)
									LOCALE_DISPLAY = value
									L = LOCALE[LOCALE_DISPLAY]
									sConfig.general.displayLocale = value
									
									cDisplay.Config.waterfall:Close('ConfigWindow')
									cDisplay.Config:Initialize()
									cDisplay.Config:Open("general")
									cDisplay.Average:Update()
								end,
								validate = {"enUS", "frFR", "deDE", "esES"},
								order = 102,
							},
						},
					},
					messages = {
						type = 'group',
						name = L["Messages Name"],
						order = 200,
						args = {
							messagesPlayerHeader = {
								type = 'header',
								name = L["Player Messages"],
								order = 201,
							},
							messagesPlayerFloating = {
								type = 'toggle',
								name = L["Show Floating"],
								get = function() return sConfig.messages.playerFloating end,
								set = function(x) sConfig.messages.playerFloating = x end,
								order = 202,
							},
							messagesPlayerChat = {
								type = 'toggle',
								name = L["Show In Chat"],
								get = function() return sConfig.messages.playerChat end,
								set = function(x) sConfig.messages.playerChat = x end,
								order = 203,
							},
							messagesMargin1 = {
								type = 'header',
								name = ' ',
								order = 204,
							},
							messagesPetHeader = {
								type = 'header',
								name = L["Pet Messages"],
								order = 205,
							},
							messagesPetFloating = {
								type = 'toggle',
								name = L["Show Floating"],
								get = function() return sConfig.messages.petFloating end,
								set = function(x) sConfig.messages.petFloating = x end,
								order = 206,
							},
							messagesPetChat = {
								type = 'toggle',
								name = L["Show In Chat"],
								get = function() return sConfig.messages.petChat end,
								set = function(x) sConfig.messages.petChat = x end,
								order = 207,
							},
						},
					},
					average = {
						type = 'group',
						name = L["Average Name"],
						order = 300,
						args = {
							averageAppearenceHeader = {
								type = 'header',
								name = L["Appearence Header"],
								order = 301,
							},
							averageVisible = {
								type = 'toggle',
								name = L["Show Average Window"],
								get = function() return sConfig.averageDisplay.visible end,
								set = function(x) sConfig.averageDisplay.visible = x; cDisplay.Average:Update(); end,
								order = 302,
							},
							averageShowHeader = {
								type = 'toggle',
								name = L["Show XToLevel Header"],
								get = function() return sConfig.averageDisplay.header end,
								set = function(x) sConfig.averageDisplay.header = x; cDisplay.Average:Update(); end,
								order = 303,
							},
							averageShowBackdrop = {
								type = 'toggle',
								name = L["Show Window Frame"],
								get = function() return sConfig.averageDisplay.backdrop end,
								set = function(x) sConfig.averageDisplay.backdrop = x; cDisplay.Average:Update(); end,
								order = 304,
							},
							marginHeader1 = {
								type = 'header',
								name = "",
								order = 305,
							},
							averageDataHeader = {
								type = 'header',
								name = L["Data Header"],
								order = 306,
							},
							averageShowKills = {
								type = 'toggle',
								name = L["Show Kills"],
								get = function() return sConfig.averageDisplay.playerKills end,
								set = function(x) sConfig.averageDisplay.playerKills = x; cDisplay.Average:Update(); end,
								order = 307,
							},
							averageShowQuests = {
								type = 'toggle',
								name = L["Show Quests"],
								get = function() return sConfig.averageDisplay.playerQuests end,
								set = function(x) sConfig.averageDisplay.playerQuests = x; cDisplay.Average:Update(); end,
								order = 308,
							},
							averageShowPet = {
								type = 'toggle',
								name = L["Show Pet Kills"],
								get = function() return sConfig.averageDisplay.petKills end,
								set = function(x) sConfig.averageDisplay.petKills = x; cDisplay.Average:Update(); end,
								order = 309,
							},
							marginHeader2 = {
								type = 'header',
								name = "",
								order = 310,
							},
							averageBehaviorHeader = {
								type = 'header',
								name = L["Behavior Header"],
								order = 311,
							},
							averageBehaviorLock = {
								type = 'toggle',
								name = L["Lock Avarage Display"],
								get = function() return not sConfig.general.allowDrag end,
								set = function(x) sConfig.general.allowDrag = not x; cDisplay.Average:Update(); end,
								order = 312,
							},
							averageBehaviorAllowClick = {
								type = 'toggle',
								name = L["Allow Click"],
								get = function() return sConfig.general.allowSettingsClick end,
								set = function(x) sConfig.general.allowSettingsClick = x; cDisplay.Average:Update(); end,
								order = 313,
							},
						},
					},
					ldb = {
						type = 'group',
						name = L["LDB Name"],
						order = 400,
						args = {
							ldbGeneralHeader = {
								type = 'header',
								name = L["LDB General Header"],
								order = 401,
							},
							ldbGeneralShowText = {
								type = 'toggle',
								name = L["Show LDB Text"],
								get = function() return sConfig.ldb.showText end,
								set = function(x) sConfig.ldb.showText = x; UpdateDataObject(); end,
								order = 402,
							},
							ldbGeneralShowLabel = {
								type = 'toggle',
								name = L["Show LDB Label"],
								get = function() return sConfig.ldb.showLabel end,
								set = function(x) sConfig.ldb.showLabel = x; UpdateDataObject(); end,
								order = 403,
							},
							ldbGeneralShowIcon = {
								type = 'toggle',
								name = L["Show LDB Icon"],
								get = function() return sConfig.ldb.showIcon end,
								set = function(x) sConfig.ldb.showIcon = x; UpdateDataObject(); end,
								order = 405,
							},
							ldbAllowTextColor = {
								type = 'toggle',
								name = L["LDB Allow Text Color"],
								get = function() return sConfig.ldb.allowTextColor end,
								set = function(x) sConfig.ldb.allowTextColor = x; UpdateDataObject(); end,
								order = 406,
							},
							marginHeader1 = {
								type = 'header',
								name = "",
								order = 409,
							},
							ldbTextHeader = {
								type = 'header',
								name = L["LDB Text Header"],
								order = 410,
							},
							ldbTextPattern = {
								type = 'text',
								name = L["LDB Text Pattern"],
								get = function() return sConfig.ldb.textPattern; end,
								set = function(value) sConfig.ldb.textPattern = value; UpdateDataObject(); end,
								validate = {"default", "minimal", "minimal_dashed", "brackets"},
								order = 411,
							},
							ldbTextVerbose = {
								type = 'toggle',
								name = L["LDB Text Verbose"],
								get = function() return sConfig.ldb.text.verbose end,
								set = function(x) sConfig.ldb.text.verbose = x; UpdateDataObject(); end,
								order = 412,
							},
							ldbTextUseColorValues = {
								type = 'toggle',
								name = L["LDB Text Value Colors"],
								get = function() return sConfig.ldb.text.colorValues end,
								set = function(x) sConfig.ldb.text.colorValues = x; UpdateDataObject(); end,
								order = 413,
							},
							marginHeader2 = {
								type = 'header',
								name = "",
								order = 415,
							},
							ldbTextKills = {
								type = 'toggle',
								name = L["Text Show Kills"],
								get = function() return sConfig.ldb.text.kills end,
								set = function(x) sConfig.ldb.text.kills = x; UpdateDataObject(); end,
								order = 416,
							},
							ldbTextQuests = {
								type = 'toggle',
								name = L["Text Show Quests"],
								get = function() return sConfig.ldb.text.quests end,
								set = function(x) sConfig.ldb.text.quests = x; UpdateDataObject(); end,
								order = 417,
							},
							ldbTextXP = {
								type = 'toggle',
								name = L["Text Show XP Progress"],
								get = function() return sConfig.ldb.text.xp end,
								set = function(x) sConfig.ldb.text.xp = x; UpdateDataObject(); end,
								order = 418,
							},
							ldbTextPet = {
								type = 'toggle',
								name = L["Text Show Pets"],
								get = function() return sConfig.ldb.text.pet end,
								set = function(x) sConfig.ldb.text.pet = x; UpdateDataObject(); end,
								order = 419,
							},
							ldbTextPetXP = {
								type = 'toggle',
								name = L["Text Show Pet XP Progress"],
								get = function() return sConfig.ldb.text.petxp end,
								set = function(x) sConfig.ldb.text.petxp = x; UpdateDataObject(); end,
								order = 419,
							},
							
							marginHeader3 = {
								type = 'header',
								name = "",
								order = 424,
							},
							ldbTooltipHeader = {
								type = 'header',
								name = L["LDB Tooltip Header"],
								order = 425,
							},
							ldbTooltipDetails = {
								type = 'toggle',
								name = L["Show Tooltip Details"],
								get = function() return sConfig.ldb.tooltip.showDetails end,
								set = function(x) sConfig.ldb.tooltip.showDetails = x; UpdateDataObject(); end,
								order = 426,
							},
							ldbTooltipExperience = {
								type = 'toggle',
								name = L["Show Tooltip Experience"],
								get = function() return sConfig.ldb.tooltip.showExperience end,
								set = function(x) sConfig.ldb.tooltip.showExperience = x; UpdateDataObject(); end,
								order = 427,
							},
						},
					},
				},
			}
		end,
    }
}